@@ -167,7 +167,9 @@ class TokenProviderBase(abc.ABC):
     async def get_token_async(self):
         """Get a token asynchronously silently from cache or authenticate if cached token is not found"""
 
-        @distributed_trace_async(name_of_span=f"{self.name()}.get_token_async", tracing_attributes=self.context_async(), kind=SpanKind.CLIENT)
+        context = await self.context_async()
+
+        @distributed_trace_async(name_of_span=f"{self.name()}.get_token_async", tracing_attributes=context, kind=SpanKind.CLIENT)
         async def _get_token_async():
             if not self.is_async:
                 raise KustoAsyncUsageError("get_token_async", self.is_async)
@@ -179,13 +181,19 @@ class TokenProviderBase(abc.ABC):
             if token is None:
                 async with self._async_lock:
                     token = await KustoTracing.call_func_tracing_async(
-                        self._get_token_impl_async, name_of_span=f"{self.name()}.get_token_impl_async", tracing_attributes=self.context_async()
+                        self._get_token_impl_async, name_of_span=f"{self.name()}.get_token_impl_async", tracing_attributes=context
                     )
 
             return self._valid_token_or_throw(token)
 
         return await _get_token_async()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
     @staticmethod
     @abc.abstractmethod
     def name() -> str:
@@ -691,3 +699,12 @@ class AzureIdentityTokenCredentialProvider(CloudInfoTokenProvider):
 
     def _get_token_from_cache_impl(self) -> Optional[dict]:
         return None
+
+    def close(self):
+        if self.credential is not None:
+            if self.is_async:
+                asyncio.get_event_loop().run_in_executor(None, self.credential.close())
+            else:
+                self.credential.close()
+            self.credential = None
+            self.credential_from_login_endpoint = None
